/**
 * RMIEventBus is an event bus implemented in RMI.
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gaugeInfrastructure/RMI/RMIEventBus.java,v 1.3 2002/01/31 16:39:03 schmerl Exp $
 * $Log: RMIEventBus.java,v $
 * Revision 1.3  2002/01/31 16:39:03  schmerl
 * Added CVS header information
 *
 */

package edu.cmu.cs.able.gaugeInfrastructure.RMI;

import java.util.Vector;
import java.util.Enumeration;
import java.rmi.*;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.RMI.Events.*;
/** The simulator of RMI event bus. This marshals and unmarshals events,
 * provides an announcement interface, as well as a request/response interface.
 * @author      Hong Yan
 * @author      Bradley Schmerl
 * @version     1.0
 * @since       JDK1.3.1 */
public class RMIEventBus extends java.rmi.server.UnicastRemoteObject implements RMIEventBusHandle, java.io.Serializable
{
    private StringObjectVector interestList;
    private StringObjectVector gaugeHandleList;
    private StringObjectVector gaugeEventList;
    private StringObjectVector gaugeHostList;
    private StringObjectVector gaugeMgrHostList;
    private StringObjectVector gaugeMgrTypeList;    
    private StringObjectVector gaugeIDList;
    private StringObjectVector gaugeMgrHandleList;
    private StringObjectVector gaugeConsumerHandleList;
    public int busPort = 1099;
    

    /** Creates a new event bus
     */    
    public RMIEventBus(int port) throws java.rmi.RemoteException
    {
        super();
        busPort = port;
        interestList = new StringObjectVector();
        gaugeHandleList = new StringObjectVector();
        gaugeEventList = new StringObjectVector();
        gaugeHostList = new StringObjectVector();
        gaugeMgrHostList = new StringObjectVector();
        gaugeMgrTypeList = new StringObjectVector();        
        gaugeIDList = new StringObjectVector();
        gaugeMgrHandleList = new StringObjectVector();
        gaugeConsumerHandleList = new StringObjectVector();
    }

    /** Configures a gauge
     * @param theGaugeID The GaugeID of the gauge to be configured
     * @param configParams The configuration parameters
     */    
    public void configureGauge(GaugeID theGaugeID, StringPairVector configParams) throws java.rmi.RemoteException
    {
        try {
            ConfiguredEvent configuredEvent = new ConfiguredEvent(theGaugeID);
            configuredEvent.configParams = configParams;
            gaugeEventList.addElement(theGaugeID.uid, configuredEvent);
            System.out.println("Gauge Event List: "+gaugeEventList.toString());
            
            for (int i = 0; i < gaugeHandleList.size(); i++) {
                if (gaugeHandleList.nameAt(i).equals(theGaugeID.uid)) {
                    System.out.println(theGaugeID.toString());
                    RMIGaugeHandleInt gh = (RMIGaugeHandleInt)gaugeHandleList.valueAt(i);
                    gh.configure(configParams);
                    reportConfigured(theGaugeID, configParams);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /** Creates a gauge
     * @param theGaugeID The GaugeID of the gauge to be created
     * @param setupParams The setup parameters
     */    
    public void createGauge(GaugeID theGaugeID, StringPairVector setupParams, StringPairVector mappings) throws java.rmi.RemoteException
    {
        try {
            for (int i = 0; i < gaugeMgrTypeList.size(); i++) {
                String gaugeMgrType = (String)gaugeMgrTypeList.valueAt(i);
                if (gaugeMgrType.equals(theGaugeID.gaugeType)) {
                    System.out.println("reqest gaugemgr to create gauge:"+theGaugeID.toString());
                    String id = gaugeMgrTypeList.nameAt(i);
                    String host = (String)gaugeMgrHostList.getValue(id);
                    RMIGaugeMgrHandle gmh = (RMIGaugeMgrHandle)gaugeMgrHandleList.getValue(id);
                    gmh.createGauge(theGaugeID, setupParams, mappings, host, Global.getHost()+":"+busPort);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /** Deletes a gauge
     * @param theGaugeID The GaugeID of the gauge to be deleted
     */    
    public void deleteGauge(GaugeID theGaugeID) throws java.rmi.RemoteException
    {
        int i;
        try {
            for (i = 0; i < gaugeMgrTypeList.size(); i++) {
                String gaugeMgrType = (String)gaugeMgrTypeList.valueAt(i);
                if (gaugeMgrType.equals(theGaugeID.gaugeType)) {
                    System.out.println("reqest gaugemgr to delete gauge:"+theGaugeID.toString());
                    String id = gaugeMgrTypeList.nameAt(i);
                    String host = (String)gaugeMgrHostList.getValue(id);
                    RMIGaugeMgrHandle gmh = (RMIGaugeMgrHandle)gaugeMgrHandleList.getValue(id);
                    gmh.deleteGauge(new RMIGauge(theGaugeID));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        reportDeleted(theGaugeID);
        /*for (i = 0; i < gaugeHandleList.size(); i++) {
            if (gaugeHandleList.nameAt(i).equals(theGaugeID.uid)) {
                RMIGaugeHandleInt gh = (RMIGaugeHandleInt)gaugeHandleList.valueAt(i);
                reportDeleted(theGaugeID);
                //gh.delete();
            }
        }*/
    }
    
    
    /** Subscribes interest
     * @param gaugeConsumerUID The UID of the gauge consumer subscribing interest
     * @param theGaugeID The GaugeID of the gauge to be observed     
     * @param gaugeEvent The gauge event in which the gauge consumer is interested
     */    
    public void subscribeInterest(String gaugeConsumerUID, GaugeID theGaugeID, GaugeEventType gaugeEvent) throws java.rmi.RemoteException
    {
        if (gaugeEvent.createdEvents){
            CreatedEvent createdEvent = new CreatedEvent(theGaugeID);
            interestList.addElement(gaugeConsumerUID, createdEvent);
        }
        if (gaugeEvent.deletedEvents){
            DeletedEvent deletedEvent = new DeletedEvent(theGaugeID);
            interestList.addElement(gaugeConsumerUID, deletedEvent);
        }
        if (gaugeEvent.configuredEvents){
            ConfiguredEvent configuredEvent = new ConfiguredEvent(theGaugeID);
            interestList.addElement(gaugeConsumerUID, configuredEvent);
        }
        if (gaugeEvent.valueEvents){
            ValueEvent valueEvent = new ValueEvent(theGaugeID);
            interestList.addElement(gaugeConsumerUID, valueEvent);
        }
        if (gaugeEvent.multipleValueEvents){
            MultipleValueEvent multipleValueEvent = new MultipleValueEvent(theGaugeID);
            interestList.addElement(gaugeConsumerUID, multipleValueEvent);
        }
        System.out.println("Interest subscribed from gaugeConsumer("+gaugeConsumerUID+") as follows:\n"+theGaugeID.toString());
        findMatch();
    }

    private boolean findMatch() throws java.rmi.RemoteException
    {
        try {
        System.err.println(gaugeHandleList.toString());
        System.err.println(gaugeConsumerHandleList.toString());
        System.err.println(interestList.toString());
        for (int k = 0; k < gaugeHandleList.size(); k++) {
            RMIGaugeHandleInt gh = (RMIGaugeHandleInt)gaugeHandleList.valueAt(k);
            GaugeID gid = gh.getGaugeID();
            System.err.println(gid.toString());
            for (int i = 0; i < interestList.size(); i++) {
                GaugeID theGaugeID = (GaugeEvent)interestList.valueAt(i);
                System.err.println(theGaugeID.getClass().getName());
                System.err.println(theGaugeID.toString());
                if (!Global.match(theGaugeID, gid)) continue;
                for (int j = 0; j < gaugeConsumerHandleList.size(); j++) {
                    RMIGaugeConsumerHandle gch = (RMIGaugeConsumerHandle)gaugeConsumerHandleList.valueAt(j);
                    //System.err.println(gch.toString());
                    if (!gch.getGaugeConsumerUID().equals(interestList.nameAt(i)))
                        continue;
                        
                    int configSize = gh.getConfigParamsSize();
                    int setupSize = gh.getSetupParamsSize();
                    System.out.println("configsize:"+configSize+" setupSize:"+setupSize);
                    StringPairVector configParams = new StringPairVector();
                    StringPairVector setupParams = new StringPairVector();
                    if (configSize <= 0 && setupSize <= 0) {
                        System.out.println("equals found:"+gid.toString());
                        gch.addGauge(gid);
                    }
                    else {
                        if (configSize > 0) configParams = gh.getConfigParams();
                        if (setupSize > 0) setupParams = gh.getSetupParams();
                        System.out.println("equals found:"+gid.toString()+configParams.toString());
                        gch.addGauge(gid, configParams, setupParams);
                    }
                }
            }
        }
    } catch (Exception e) {
        e.printStackTrace();
    }
        return true;
    }

    /** Gets the host of a gauge
     * @param gid The GaugeID of the gauge to be queried for host address
     * @return The host of the gauge in the form of address:port     
     */    
    public String getGaugeHost(GaugeID gid) throws java.rmi.RemoteException
    {
        for (int i = 0; i < gaugeIDList.size(); i++) {
            GaugeID theGaugeID = (GaugeID)gaugeIDList.valueAt(i);
            System.err.println("getGaugeHost:"+theGaugeID.toString()+"gid:"+gid.toString());
            if (theGaugeID.equals(gid))
                return (String)gaugeHostList.getValue(theGaugeID.uid);
        }
        return null;
    }

    /** Registers a gauge manager
     * @param gaugeMgrHost The host of the gauge manager to be registered
     * @param gaugeMgrUID The UID of the gauge manager to be registered
     * @param gaugeMgrUID The type of the gauge manager to be registered
     */    
    public void registerGaugeMgr(String gaugeMgrHost, String gaugeMgrUID, String gaugeMgrType) throws java.rmi.RemoteException
    {
        System.out.println(gaugeMgrHost);
        RMIGaugeMgrHandle gmh = (RMIGaugeMgrHandle)connectTo(gaugeMgrHost, gaugeMgrType);
        gaugeMgrHandleList.addElement(gaugeMgrUID, gmh);
        gaugeMgrHostList.addElement(gaugeMgrUID, gaugeMgrHost);
        gaugeMgrTypeList.addElement(gaugeMgrUID, gaugeMgrType);     
        System.out.println(gaugeMgrHandleList.toString());
    }

    /** Get a gauge manager
     * @param gaugeMgrType The type of the gauge manager
     * @return The UID of the gauge manager
     */    
    public String getGaugeMgr(String gaugeMgrType) throws java.rmi.RemoteException
    {
        System.out.println(gaugeMgrType);
        for (int i = 0; i < gaugeMgrTypeList.size(); i++) {
            String mgrType = (String)gaugeMgrTypeList.valueAt(i);
            if (mgrType.equals(gaugeMgrType)) {
                return (String)gaugeMgrTypeList.nameAt(i);
            }
        }
        return "";
    }
    
    public StringPairVector queryConfigMetaInfo(String gaugeType) throws java.rmi.RemoteException
    {
        int i;
        try {
            for (i = 0; i < gaugeMgrTypeList.size(); i++) {
                String gaugeMgrType = (String)gaugeMgrTypeList.valueAt(i);
                if (gaugeMgrType.equals(gaugeType)) {
                    System.out.println("request gaugemgr for meta info");
                    String id = gaugeMgrTypeList.nameAt(i);
                    String host = (String)gaugeMgrHostList.getValue(id);
                    RMIGaugeMgrHandle gmh = (RMIGaugeMgrHandle)gaugeMgrHandleList.getValue(id);
                    StringPairVector spv = gmh.queryConfigMetaInfo(gaugeType);
                    System.out.println(spv.toString());
                    return spv;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }   

    public StringPairVector queryValueMetaInfo(String gaugeType) throws java.rmi.RemoteException
    {
        int i;
        try {
            for (i = 0; i < gaugeMgrTypeList.size(); i++) {
                String gaugeMgrType = (String)gaugeMgrTypeList.valueAt(i);
                if (gaugeMgrType.equals(gaugeType)) {
                    System.out.println("request gaugemgr for meta info");
                    String id = gaugeMgrTypeList.nameAt(i);
                    String host = (String)gaugeMgrHostList.getValue(id);
                    RMIGaugeMgrHandle gmh = (RMIGaugeMgrHandle)gaugeMgrHandleList.getValue(id);
                    StringPairVector spv = gmh.queryValueMetaInfo(gaugeType);
                    System.out.println(spv.toString());
                    return spv;
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }   

    /** Registers a gauge
     * @param theGaugeID The GaugeID of the gauge to be registered
     * @param setupParams The setup parameters of the gauge to be registered
     * @param gaugeHost The host of the gauge to be registered in the form of address:port
     */    
    public void registerGauge(GaugeID theGaugeID, StringPairVector setupParams, String gaugeHost) throws java.rmi.RemoteException
    {
        RMIGaugeHandleInt gh = (RMIGaugeHandleInt) connectTo(gaugeHost, "Gauge");
        gaugeHandleList.addElement(theGaugeID.uid, gh);
        gaugeIDList.addElement(theGaugeID.uid, theGaugeID);
        gaugeHostList.addElement(theGaugeID.uid, gaugeHost);
        CreatedEvent createdEvent = new CreatedEvent(theGaugeID);
        gaugeEventList.addElement(theGaugeID.uid, createdEvent);
        System.out.println("register Gauge: "+theGaugeID.toString());       
        reportCreated(theGaugeID);      
    }

    /** Unregisters a gauge
     * @param gaugeID The GaugeID of the gauge to be unregistered
     */    
    private void unregisterGauge(GaugeID gaugeID)
    {
        gaugeHandleList.removeElement(gaugeID.uid);
        DeletedEvent deletedEvent = new DeletedEvent(gaugeID);
        gaugeEventList.addElement(gaugeID.uid, deletedEvent);
        gaugeHostList.removeElement(gaugeID.uid);
        gaugeIDList.removeElement(gaugeID.uid);
        System.out.println("Gauge Event List: "+gaugeEventList.toString());
    }

    /** Registers a gauge consumer
     * @param gaugeConsumerUID The UID of the gauge consumer to be registered
     * @param gaugeConsumerHost The host of the gauge consumer to be registered in the form of address:port
     */    
    public void registerGaugeConsumer(String gaugeConsumerUID, String gaugeConsumerHost) throws java.rmi.RemoteException
    {
        RMIGaugeConsumerHandle gch = (RMIGaugeConsumerHandle)connectTo(gaugeConsumerHost, "GaugeConsumer");
        gaugeConsumerHandleList.addElement(gaugeConsumerUID, gch);
        System.out.println(gaugeConsumerHandleList.toString());     
    }
    
    /** Connects to a RMI service
     * @param host The host of the service
     * @param service The name of the service
     */    
    public java.rmi.Remote connectTo(String host, String service) {
        try {
            /*System.setSecurityManager (new RMISecurityManager() {
                public void checkConnect (String host, int port) {}
                public void checkConnect (String host, int port, Object context) {}
            });*/

            if (service.equals("GaugeConsumer")) {
                RMIGaugeConsumerHandle gch = (RMIGaugeConsumerHandle)Naming.lookup("rmi://"+host+"/"+service);
                //gaugeConsumerHandleList.addElement(gch);
                return gch;
            } else if (service.equals("Gauge")) {
                RMIGaugeHandleInt gh = (RMIGaugeHandleInt)Naming.lookup("rmi://"+host+"/"+service);
                return gh;
            } else {
                RMIGaugeMgrHandle gmh = (RMIGaugeMgrHandle)Naming.lookup("rmi://"+host+"/"+service);
                return gmh;
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /** Reports a value
     * @param gauge The GaugeID of the gauge reporting the value
     * @param valueName The name of the value
     * @param propertyName The property name of the value
     * @param value The content of the value     
     */    
    public boolean reportValue(GaugeID gauge, String valueName, String propertyName, String value) throws java.rmi.RemoteException
    {
        System.out.println("value reported from"+gauge.toString());
        for (int i = 0; i < interestList.size(); i++) {
            GaugeID theGaugeID = (GaugeEvent)interestList.valueAt(i);
            if (!interestList.valueAt(i).getClass().getName().equals("edu.cmu.cs.able.gaugeInfrastructure.Events.ValueEvent") || !Global.match(theGaugeID, gauge)) continue;
            RMIGaugeHandleInt gh = (RMIGaugeHandleInt)gaugeHandleList.getValue(gauge.uid);
            for (int j = 0; j < gaugeConsumerHandleList.size(); j++) {
                RMIGaugeConsumerHandle gch = (RMIGaugeConsumerHandle)gaugeConsumerHandleList.valueAt(j);
                if (!gch.getGaugeConsumerUID().equals(interestList.nameAt(i)))
                    continue;
                System.out.println("equals found for "+gh.getGaugeID());
                gch.onReportValue(gh.getGaugeID(), valueName, propertyName, value);
            }
        }
        return true;
    }
    
    
    public boolean reportMultipleValues(GaugeID gauge, GaugeValueVector values) throws java.rmi.RemoteException
    {
        return true;
    }
    
    /** Reports a configured event
     * @param gauge The GaugeID of the gauge reporting the event
     * @param configParams The configuration parameters in the report
     */    
    public boolean reportConfigured(GaugeID gauge, StringPairVector configParams) throws java.rmi.RemoteException
    {
        for (int i = 0; i < interestList.size(); i++) {
            GaugeID theGaugeID = (GaugeEvent)interestList.valueAt(i);
            if (!interestList.valueAt(i).getClass().getName().equals("edu.cmu.cs.able.gaugeInfrastructure.Events.ConfiguredEvent") || !Global.match(theGaugeID, gauge)) continue;
            RMIGaugeHandleInt gh = (RMIGaugeHandleInt)gaugeHandleList.getValue(gauge.uid);
            for (int j = 0; j < gaugeConsumerHandleList.size(); j++) {
                RMIGaugeConsumerHandle gch = (RMIGaugeConsumerHandle)gaugeConsumerHandleList.valueAt(j);
                if (!gch.getGaugeConsumerUID().equals(interestList.nameAt(i)))
                    continue;
                System.out.println("equals found");
                gch.onReportConfigured(gauge, configParams);
            }
        }
        return true;
    }
    
    /** Reports a deleted event
     * @param gauge The GaugeID of the gauge reporting the event
     */    
    public boolean reportDeleted(GaugeID gauge) throws java.rmi.RemoteException
    {
        for (int i = 0; i < interestList.size(); i++) {
            GaugeID theGaugeID = (GaugeEvent)interestList.valueAt(i);
            if (!interestList.valueAt(i).getClass().getName().equals("edu.cmu.cs.able.gaugeInfrastructure.Events.DeletedEvent") || !Global.match(theGaugeID, gauge)) continue;
            RMIGaugeHandleInt gh = (RMIGaugeHandleInt)gaugeHandleList.getValue(gauge.uid);
            for (int j = 0; j < gaugeConsumerHandleList.size(); j++) {
                RMIGaugeConsumerHandle gch = (RMIGaugeConsumerHandle)gaugeConsumerHandleList.valueAt(j);
                if (!gch.getGaugeConsumerUID().equals(interestList.nameAt(i)))
                    continue;
                System.out.println("equals found");
                gch.onReportDeleted(gauge);
            }
        }
        unregisterGauge(gauge);     
        return true;
    }

    /** Reports a created event
     * @param gauge The GaugeID of the gauge reporting the event
     */    
    public boolean reportCreated(GaugeID gauge) throws java.rmi.RemoteException
    {
        try {
            RMIGaugeHandleInt gh = (RMIGaugeHandleInt)gaugeHandleList.getValue(gauge.uid);
            for (int i = 0; i < interestList.size(); i++) {
                GaugeID theGaugeID = (GaugeEvent)interestList.valueAt(i);
                if (!theGaugeID.getClass().getName().equals("edu.cmu.cs.able.gaugeInfrastructure.Events.CreatedEvent") || !Global.match(theGaugeID, gauge)) continue;
                for (int j = 0; j < gaugeConsumerHandleList.size(); j++) {
                    RMIGaugeConsumerHandle gch = (RMIGaugeConsumerHandle)gaugeConsumerHandleList.valueAt(j);
                    if (!gch.getGaugeConsumerUID().equals(interestList.nameAt(i)))
                        continue;
                    int configSize = gh.getConfigParamsSize();
                    int setupSize = gh.getSetupParamsSize();
                    System.out.println("configsize:"+configSize+" setupSize:"+setupSize);
                    StringPairVector configParams = new StringPairVector();
                    StringPairVector setupParams = new StringPairVector();
                    if (configSize <= 0 && setupSize <= 0) {
                        System.out.println("equals found:"+theGaugeID.toString());
                        gch.addGauge(gauge);
                    }
                    else {
                        if (configSize > 0) configParams = gh.getConfigParams();
                        if (setupSize > 0) setupParams = gh.getSetupParams();
                        System.out.println("equals found:"+theGaugeID.toString()+configParams.toString());
                        gch.addGauge(gauge, configParams, setupParams);
                    }
                }
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }

    
    public void subscribeToGaugeEvent(GaugeEvent event, Subscriber subscriber) throws java.rmi.RemoteException
    {
    }
    
    public void unsubscribeToGaugeEvent(GaugeEvent event, Subscriber subscriber) throws java.rmi.RemoteException
    {
    }

}
