/*
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gaugeInfrastructure/RMI/RMIGauge.java,v 1.7 2002/05/24 18:09:25 schmerl Exp $
 * $Log: RMIGauge.java,v $
 * Revision 1.7  2002/05/24 18:09:25  schmerl
 * Moved GaugeImpl to RMI Gauge, and implemented gauge methods to delegate to GaugeImpl here, rather than require it in all gauge implementations.
 *
 * Revision 1.6  2002/02/20 20:41:27  schmerl
 * 1. Fixed gauge deletion in the Siena version so that deletes don't timeout.
 * 2. Changed gauge implementation pattern so that gauge computation does not need to be included in both RMII and Siena versions.
 *
 * Revision 1.5  2002/01/31 16:39:03  schmerl
 * Added CVS header information
 *
 */
package edu.cmu.cs.able.gaugeInfrastructure.RMI;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;

/** RMIGauge.java
 * @author      Hong Yan
 * @author      Bradley Schmerl
 * @version     1.0
 * @since       JDK1.3.1
 * @ChangeLog:
 *   1/29/2002 BRS Added mappings table to store mappings
 *                  between values and the architecture. Also
 *                  added associated methods (xferMappings)
 *    1/29/2002 BRS Added independent reporting thread and
 *                  protected members doReporting and sleepTime
 *                  that control this thread.
 *
 */
public class RMIGauge implements GaugeControl, java.io.Serializable {
    protected RMIGaugeReportingBus gaugeBus;
    protected String gaugeType = "edu.cmu.cs.able.gaugeInfrastructure.RMI.genericGauge";
    protected String creatorID = null;
 
    protected GaugeImpl impl = null;
    
    // The following members contain information about the gauge
    public StringPairVector setupParams = null;
    public StringPairVector configParams = new StringPairVector ();
    protected StringPairVector mappings = new StringPairVector ();    
    protected GaugeID gaugeID;
    protected static String busHost = "localhost:1099";
    private String host = null;
    

    /** Creates new RMIGauge */
    public RMIGauge (GaugeID gaugeID) {
        try {
            this.gaugeID = gaugeID;
            setupParams = new StringPairVector ();
            configParams = new StringPairVector ();
        }
        catch (Exception e) {}
    }
    
    /** Creates new RMIGauge */
    public RMIGauge (String modelName, String modelType, String gaugeName, String creatorID) {
        try {
            gaugeID = new GaugeID ();
            gaugeBus = new RMIGaugeReportingBus (busHost);
            //gaugeID.uid = Global.generateUniqueID ();
            gaugeID.modelName = modelName;
            gaugeID.modelType = modelType;
            gaugeID.gaugeName = gaugeName;
            gaugeID.gaugeType = gaugeName;
            setupParams = new StringPairVector ();
            configParams = new StringPairVector ();
            StringPairVector mappings = new StringPairVector ();
        }
        catch (Exception e) {}
    }

    /** Creates new RMIGauge */
    public RMIGauge (GaugeID gaugeID, String creatorID, StringPairVector setupParams, StringPairVector mappings) {
        try {
            if (gaugeID != null)
                this.gaugeID = gaugeID;
            else this.gaugeID = new GaugeID ();
            gaugeBus = new RMIGaugeReportingBus (busHost);
            //this.gaugeID.uid = Global.generateUniqueID ();
            this.setupParams = new StringPairVector();
            if (setupParams != null) setupParams.copyInto (this.setupParams);
            this.configParams = new StringPairVector();
            this.init ();
        }
        catch (Exception e) {
            e.printStackTrace ();
        }
    }
    
    /** Initializes the RMIGauge */
    public void init () {
        try {
            host = Initialization.startGaugeService (this);
            //gaugeBus.registerGauge (gaugeID, setupParams, host);
        }
        catch (Exception e) {
            e.printStackTrace ();
        }
    }

    /** Register the RMIGauge */
    public void register () {
        try {
            gaugeBus.registerGauge (gaugeID, setupParams, host);
        }
        catch (Exception e) {
            e.printStackTrace ();
        }
    }
    
    /** Deletes the RMIGauge */
    public void delete () {
        try {
            Runtime.getRuntime ().exit (0);
        }
        catch (Exception e) {
            //e.printStackTrace();
        }
    }
    
    /** Returns the gauge ID of this gauge.
     * @return
     */
    public GaugeID getGaugeID () {
        return gaugeID;
    }
    
    /** Returns the configuration parameters of this gauge.
     * @return
     */
    public StringPairVector getConfigParams () {
        return configParams;
    }

    /** Returns the setup parameters of this gauge.
     * @return
     */
    public StringPairVector getSetupParams () {
        return setupParams;
    }
    
    public int getConfigParamsSize () {
        if (configParams != null)
            return configParams.size ();
        else return 0;
    }

    public int getSetupParamsSize () {
        if (setupParams != null)
            return setupParams.size ();
        else return 0;
    }
    
    // These methods must be overridden by particular gauges
    
    /** Configures the gauge.
     * @param configParams The configuration parameters for this gauge.
     * @return
     */
    public boolean configure (StringPairVector configParams) {
	if (impl != null) 
		return impl.configure (configParams);
	else
        	return false;
    };
    
    /** Returns all the values that this gauge reports.
     * @param values
     * @return
     */
    public boolean queryAllValues (GaugeValueVector values) {
	if (impl != null) 
		return impl.queryAllValues (values);
	else
        	return false;
    }
    
    /** Returns the state of this gauge.
     * @param setupParams
     * @param configParams
     * @param mappings
     * @return
     */
    public boolean queryState (StringPairVector setupParams, StringPairVector configParams, StringPairVector mappings) {
	if (impl != null) 
		return impl.queryState (setupParams, configParams, mappings);
	else
        	return false;
    }
    
    /** Returns the value of a particular value name
     * @param valueName
     * @return
     */
     public String queryValue (String valueName) {
	 if (impl != null) 
		return impl.queryValue (valueName);
	 else
         	return "";
     }

 
}
