/**
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gaugeInfrastructure/RMI/RMIGaugeManagerHandle.java,v 1.1 2002/01/31 16:39:05 schmerl Exp $
 * $Log: RMIGaugeManagerHandle.java,v $
 * Revision 1.1  2002/01/31 16:39:05  schmerl
 * Added CVS header information
 *
 */

package edu.cmu.cs.able.gaugeInfrastructure.RMI;

import java.util.Vector;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;
/**
 * RMIGaugeManagerHandle is a proxy gauge manager that acts as if it is the actual
 * gauge manager it refers to. It is used to hide the details of how to maintain the
 * peer-to-peer interaction between a local caller (or client) and a
 * remote callee (or server).
 * @author      Hong Yan
 * @author      Bradley Schmerl
 * @version     1.0
 * @since       JDK1.3.1
 */
public class RMIGaugeManagerHandle implements GaugeMgr
{
    /** The reporting bus that gauges report through.
     */    
    protected RMIGaugeReportingBus gaugeBus;
    protected String mgrID = null;
    protected String mgrType = null;
    protected static String busHost = "localhost:1099";

    /**
    * Constructor of RMIGaugeConsumer
    */
    public RMIGaugeManagerHandle(String host, String gaugeType)
    {
        try {
            busHost = host;
            gaugeBus = new RMIGaugeReportingBus (host);
            mgrType = gaugeType;
            mgrID = gaugeBus.getGaugeMgr(gaugeType);
            System.out.println(mgrID);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }

    /** Each gauge manager has a unique ID.
     * @return The identifier of the gauge manager.
     */    
    public String getGaugeMgrID ()
    {
        return mgrID;
    }
    
    
    /** Creates a new gauge with the information specified.
     * @param gauge The ID of the gauge to be created.
     * @param setupParams The parameters with which to set up the gauge.
     * @param mappings The mappings of values to model properties for how
     * the gauge should report values.
     * @return A new {@link GaugeControl} interface. Note that the {@link GaugeID} of this GaugeControl may be different to the {@link GaugeID} passed in as paramters.
     * In particular, the modelName and uid may be different.
     */    
    public GaugeControl createGauge(GaugeID gauge, StringPairVector setupParams, StringPairVector mappings)
    {
        try {
            gaugeBus.createGauge(gauge, setupParams, mappings);
            RMIGauge theGauge = new RMIGauge(gauge);
            return theGauge;
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    
    /** Determines whether this gauge manager can manage gauges of
     * a particular type.
     * @param gaugeType The type of the gauge being queried
     * @return <B>true</B> if the gauge manager managers this type; <B>false</B> otherwise.
     */    
    public boolean managesType(String gaugeType)
    {
        return mgrType.equals(gaugeType);
    }
    
    
    /** Deletes a gauge. A gauge will send out a final delete announcement
     * so that all consumers can be informed.
     * @param gauge The gauge to be deleted.
     * @return <B>true</B> if the gauge was deleted; <B>false</B> otherwise.
     */    
    public boolean deleteGauge(GaugeControl gauge)
    {
        try {
            gaugeBus.deleteGauge(gauge.getGaugeID());
            System.out.println("gaugeBus.deleteGauge"+gauge.getGaugeID().toString());
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }
    
    
    /** Fills in the type information for a gauge type.
     * @param gaugeType The type of the gauge for which information is being queried.
     * @param configParamsMeta Filled in with the parameters that can be used to configure a gauge as <I>(name, type)</I> pairs.
     * @param valuesMeta Filled in with the values that the gauge can report, as <I>(name, type)</I> pairs.
     * @return <B>true</B> if the information could be determined; <B>false</B> otherwise.
     */    
    public boolean queryMetaInfo(String gaugeType, StringPairVector configParamsMeta, StringPairVector valuesMeta)
    {
        try {
            gaugeBus.queryMetaInfo(gaugeType, configParamsMeta, valuesMeta);
            System.out.println(configParamsMeta.toString());
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }
}
