/**
 * $Header: D:/Projects/Repository/GaugeAPI/edu/cmu/cs/able/gaugeInfrastructure/RMI/RMIGaugeMgrHandleImp.java,v 1.3 2002/01/31 16:39:05 schmerl Exp $
 * $Log: RMIGaugeMgrHandleImp.java,v $
 * Revision 1.3  2002/01/31 16:39:05  schmerl
 * Added CVS header information
 *
 */

package edu.cmu.cs.able.gaugeInfrastructure.RMI;

import java.util.Vector;
import java.util.Enumeration;
import java.rmi.*;

import edu.cmu.cs.able.gaugeInfrastructure.*;
import edu.cmu.cs.able.gaugeInfrastructure.util.*;
import edu.cmu.cs.able.gaugeInfrastructure.Events.*;

/**
  * RMIGaugeMgrHandleImp is the implementation of RMIGaugeMgrHandle.
 * @author      Hong Yan
 * @author      Bradley Schmerl
 * @version     1.0
 * @since       JDK1.3.1
 */
public class RMIGaugeMgrHandleImp extends java.rmi.server.UnicastRemoteObject implements RMIGaugeMgrHandle, java.io.Serializable
{
    private StringObjectVector gaugeHandleList;
    private Vector gaugeMgrHandleList;
    private StringObjectVector gaugeConsumerHandleList;
    
    private RMIGaugeMgr gaugeMgr = null;
    
    /**
    * Constructor of RmIGaugeMgrHandleImp
    * @param gaugeMgr The gauge manager this handle refers to
    */
    public RMIGaugeMgrHandleImp(RMIGaugeMgr gaugeMgr) throws java.rmi.RemoteException
    {
        super();
        gaugeHandleList = new StringObjectVector();
        gaugeConsumerHandleList = new StringObjectVector();
        this.gaugeMgr = gaugeMgr;
    }

    public String getGaugeMgrUID() throws java.rmi.RemoteException
    {
        return gaugeMgr.getGaugeMgrID();
    }
    
    public void configureGauge(GaugeID theGaugeID, StringPairVector configParams) throws java.rmi.RemoteException
    {
        for (int i = 0; i < gaugeHandleList.size(); i++) {
            if (gaugeHandleList.nameAt(i).equals(theGaugeID.uid)) {
                RMIGaugeHandleInt gh = (RMIGaugeHandleInt)gaugeHandleList.valueAt(i);
                gh.configure(configParams);
            }
        }
    }
    

    private void unregisterGauge(String gaugeUID)
    {
        gaugeHandleList.removeElement(gaugeUID);
        System.out.println(gaugeHandleList.toString());
    }

    public void createGauge(GaugeID theGaugeID, StringPairVector setupParams, StringPairVector mappings, String gaugeMgrHost, String eventBusHost) throws java.rmi.RemoteException
    {
        gaugeMgr.createGauge(theGaugeID, setupParams, mappings);//gaugeMgrHost, eventBusHost);      
    }


    public java.rmi.Remote connectTo(String host, String service) {
        try {
            /*System.setSecurityManager (new RMISecurityManager() {
                public void checkConnect (String host, int port) {}
                public void checkConnect (String host, int port, Object context) {}
            });*/

            if (service.equals("GaugeConsumer")) {
                RMIGaugeConsumerHandle gch = (RMIGaugeConsumerHandle)Naming.lookup("rmi://"+host+"/"+service);
                //gaugeConsumerHandleList.addElement(gch);
                return gch;
            } else if (service.equals("Gauge")) {
                RMIGaugeHandleInt gh = (RMIGaugeHandleInt)Naming.lookup("rmi://"+host+"/"+service);
                gaugeHandleList.addElement(gh.getGauge().getGaugeID().uid, gh);
                return gh;
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public GaugeControl createGauge(GaugeID gauge, StringPairVector setupParams, StringPairVector mappings) throws java.rmi.RemoteException
    {
        return gaugeMgr.createGauge(gauge, setupParams, mappings);
    }
    
    
    public boolean managesType(String gaugeType) throws java.rmi.RemoteException
    {
        return gaugeMgr.managesType(gaugeType);
    }
    
    
    public boolean deleteGauge(GaugeControl gauge) throws java.rmi.RemoteException
    {
        System.out.println("i'm going to delete gauge"+gauge.getGaugeID().toString());
        unregisterGauge(gauge.getGaugeID().uid);
        return gaugeMgr.deleteGauge(gauge);
    }
    
    
    public StringPairVector queryConfigMetaInfo(String gaugeType) throws java.rmi.RemoteException
    {
        StringPairVector configParamsMeta = new StringPairVector();
        StringPairVector valuesMeta = new StringPairVector();
        gaugeMgr.queryMetaInfo(gaugeType, configParamsMeta, valuesMeta);
        System.out.println(configParamsMeta.toString());
        return configParamsMeta;
    }

    public StringPairVector queryValueMetaInfo(String gaugeType) throws java.rmi.RemoteException
    {
        StringPairVector configParamsMeta = new StringPairVector();
        StringPairVector valuesMeta = new StringPairVector();
        gaugeMgr.queryMetaInfo(gaugeType, configParamsMeta, valuesMeta);
        System.out.println(valuesMeta.toString());
        return valuesMeta;
    }
    
}
